/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.io.ByteArrayOutputStream;
import java.io.StringWriter;
import java.lang.ref.WeakReference;
import java.util.Vector;

import org.apache.commons.httpclient.Header;
import org.w3c.dom.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.support.wss.IncomingWss;
import com.eviware.soapui.impl.wsdl.support.wss.WssContainer;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.propertyexpansion.PropertyExpansionContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.StringUtils;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * Simple response to a request
 * 
 * @author ole.matzura
 */

final class SinglePartHttpResponse implements WsdlResponse
{
	private final WeakReference<WsdlRequest> wsdlRequest;
	@SuppressWarnings("unused")
	private final TimeablePostMethod postMethod;
	private long timeTaken;
	private String responseContent;
	private StringToStringMap requestHeaders;
	private StringToStringMap responseHeaders;
	private final String requestContent;
	private boolean prettyPrint;
	private SSLInfo sslInfo;
	private long timestamp;
	private long responseSize;
	private Vector wssResult;
	private byte[] requestData;
	private byte[] responseBody;
	
	public SinglePartHttpResponse(WsdlRequest wsdlRequest, TimeablePostMethod postMethod, String requestContent, PropertyExpansionContext context )
	{
		this.wsdlRequest = new WeakReference<WsdlRequest>(wsdlRequest);
		this.postMethod = postMethod;
		this.requestContent = requestContent;
		this.timeTaken = postMethod.getTimeTaken();
		this.sslInfo = postMethod.getSSLInfo();
		this.timestamp = System.currentTimeMillis();
		
		// read response immediately since we need to release connection
		Settings settings = wsdlRequest.getSettings();
		
		try
		{
			responseBody = postMethod.getResponseBody();
			responseSize = responseBody.length;
			if (settings.getBoolean(HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN))
				timeTaken = postMethod.getTimeTakenUntilNow();
			
			String charset = postMethod.getResponseCharSet();
			if( charset == null )
				charset = wsdlRequest.getEncoding();
			
			charset = StringUtils.unquote( charset );
			
			responseContent = responseBody.length == 0 ? null : charset == null ? new String(responseBody) : new String(
					responseBody, charset);
			
			processIncomingWss( wsdlRequest, context );
			
			prettyPrint = wsdlRequest.getSettings().getBoolean( WsdlSettings.PRETTY_PRINT_RESPONSE_MESSAGES );
			
			initHeaders(postMethod);
			
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			postMethod.getRequestEntity().writeRequest( out );
			requestData = out.toByteArray();
		}
		catch( Exception e )
		{
			SoapUI.logError( e );
		}
	}

	private void processIncomingWss( WsdlRequest wsdlRequest, PropertyExpansionContext context )
	{
		WssContainer wssContainer = wsdlRequest.getOperation().getInterface().getProject().getWssContainer();
		IncomingWss incomingWss = wssContainer.getIncomingByName( wsdlRequest.getIncomingWss() );
		if( incomingWss != null )
		{
			try
			{
				Document document = XmlUtils.parseXml( responseContent );
				wssResult = incomingWss.processIncoming( document, context );
				if( wssResult != null && wssResult.size() > 0 )
				{
					StringWriter writer = new StringWriter();
					XmlUtils.serializePretty( document, writer );
					responseContent = writer.toString();
				}
			}
			catch( Exception e )
			{
				if( wssResult != null )
					wssResult = new Vector();
				wssResult.add( e );
			}
		}
	}
	
	private void initHeaders(TimeablePostMethod postMethod)
	{
		requestHeaders = new StringToStringMap();
		Header[] headers = postMethod.getRequestHeaders();
		for( Header header : headers )
		{
			requestHeaders.put( header.getName(), header.getValue() );
		}
		
		responseHeaders = new StringToStringMap();
		headers = postMethod.getResponseHeaders();
		for( Header header : headers )
		{
			responseHeaders.put( header.getName(), header.getValue() );
		}
		
		responseHeaders.put( "#status#", postMethod.getStatusLine().toString() );
	}

	public String getContentAsString()
	{
		if( prettyPrint )
		{
			responseContent = XmlUtils.prettyPrintXml( responseContent );
			prettyPrint = false;
		}
		
		return responseContent;
	}

	public long getContentLength()
	{
		return responseSize;
	}

	public WsdlRequest getRequest()
	{
		return wsdlRequest.get();
	}

	public long getTimeTaken()
	{
		return timeTaken;
	}

	public Attachment[] getAttachments()
	{
		return new Attachment[0];
	}

	public StringToStringMap getRequestHeaders()
	{
		return requestHeaders;
	}

	public StringToStringMap getResponseHeaders()
	{
		return responseHeaders;
	}

	public Attachment[] getAttachmentsForPart(String partName)
	{
		return new Attachment[0];
	}

	public String getRequestContent()
	{
		return requestContent;
	}

	public void setResponseContent(String responseContent)
	{
		String oldContent = this.responseContent;
		this.responseContent = responseContent;
		
		getRequest().notifyPropertyChanged( WsdlRequest.RESPONSE_CONTENT_PROPERTY, oldContent, responseContent );
	}

	public SSLInfo getSSLInfo()
	{
		return sslInfo;
	}

	public long getTimestamp()
	{
		return timestamp;
	}

	public Vector getWssResult()
	{
		return wssResult;
	}

	public byte[] getRawRequestData()
	{
		return requestData;
	}

	public byte[] getRawResponseData()
	{
		return responseBody;
	}
}