/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.MutableTestPropertyHolder;
import com.eviware.soapui.impl.wsdl.support.wsdl.UrlWsdlLoader;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.testsuite.TestProperty;
import com.eviware.soapui.model.testsuite.TestPropertyListener;

public class MapTestPropertyHolder implements MutableTestPropertyHolder
{
	private Map<String,TestProperty> propertyMap = new HashMap<String, TestProperty>();
	private Set<TestPropertyListener> listeners = new HashSet<TestPropertyListener>();
	public ModelItem modelItem;

	public MapTestPropertyHolder( ModelItem modelItem )
	{
		this.modelItem = modelItem;
	}

	private void firePropertyAdded( String name )
	{
		TestPropertyListener [] listenersArray = listeners.toArray( new TestPropertyListener[listeners.size()] );
		for( TestPropertyListener listener : listenersArray )
		{
			listener.propertyAdded( name );
		}
	}
	
	private void firePropertyRemoved( String name )
	{
		TestPropertyListener [] listenersArray = listeners.toArray( new TestPropertyListener[listeners.size()] );
		for( TestPropertyListener listener : listenersArray )
		{
			listener.propertyRemoved( name );
		}
	}
	
	private void firePropertyRenamed( String oldName, String newName )
	{
		TestPropertyListener [] listenersArray = listeners.toArray( new TestPropertyListener[listeners.size()] );
		for( TestPropertyListener listener : listenersArray )
		{
			listener.propertyRenamed( oldName, newName );
		}
	}
	
	private void firePropertyValueChanged( String name, String oldValue, String newValue )
	{
		TestPropertyListener [] listenersArray = listeners.toArray( new TestPropertyListener[listeners.size()] );
		for( TestPropertyListener listener : listenersArray )
		{
			listener.propertyValueChanged(name, oldValue, newValue );
		}
	}

	public TestProperty addProperty( String name )
	{
		TestProperty result = new InternalTestProperty( name, null );
		propertyMap.put( name.toUpperCase(), result );
		firePropertyAdded( name );
		return result;
	}

	public void addTestPropertyListener( TestPropertyListener listener )
	{
		listeners.add( listener );
	}

	public TestProperty getProperty( String name )
	{
		return propertyMap.get( name.toUpperCase() );
	}

	public String[] getPropertyNames()
	{
		return propertyMap.keySet().toArray( new String[propertyMap.size()] );
	}

	public String getPropertyValue( String name )
	{
		TestProperty property = getProperty( name );
		return property == null ? null : property.getValue();
	}

	public void removeProperty( String propertyName )
	{
		TestProperty property = getProperty( propertyName );
		if( property != null )
		{
			propertyMap.remove( propertyName.toUpperCase() );
			firePropertyRemoved( propertyName );
		}
	}

	public void removeTestPropertyListener( TestPropertyListener listener )
	{
		listeners.remove( listener );
	}

	public void setPropertyValue( String name, String value )
	{
		InternalTestProperty property = ( InternalTestProperty ) getProperty( name );
		if( property != null )
			property.setValue( value );
	}

	public boolean renameProperty( String name, String newName )
	{
		if( getProperty( newName ) != null )
			return false;
		
		InternalTestProperty property = ( InternalTestProperty ) getProperty( name );
		if( property == null )
			return false;
		
		property.setName( newName );
		return true;
	}

	/**
	 * Internal property class
	 * 
	 * @author ole
	 */
	
	public class InternalTestProperty implements TestProperty
	{
		private String name;
		private String value;

		public InternalTestProperty(String name, String value)
		{
			this.name = name;
			this.value = value;
		}

		public String getName()
		{
			return name;
		}
		
		public void setName( String name )
		{
			String oldName = getName();
			
			propertyMap.remove( oldName.toUpperCase() );
			propertyMap.put( name.toUpperCase(), this );
			
			this.name = name;
			
			firePropertyRenamed( oldName, name );
		}

		public String getDescription()
		{
			return null;
		}

		public String getValue()
		{
			return value;
		}

		public void setValue(String value)
		{
			String oldValue = getValue();
			this.value = value;
			
			firePropertyValueChanged( getName(), oldValue, value );
		}

		public boolean isReadOnly()
		{
			return false;
		}

		public Type getType()
		{
			return Type.STRING;
		}

		public ModelItem getModelItem()
		{
			return modelItem;
		}
	}

	public void saveTo( Properties props )
	{
		int cnt = 0;
		for( TestProperty p : propertyMap.values() )
		{
			String name = p.getName();
			String value = p.getValue();
			if( value == null )
				value = "";
			
			props.setProperty( name, value );
			cnt++;
		}
	}

	public Map<String, TestProperty> getProperties()
	{
		Map<String,TestProperty> result = new HashMap<String,TestProperty>();
		for( String name : propertyMap.keySet() )
			result.put( name, propertyMap.get( name ));
		
		return result;
	}

	public boolean hasProperty( String name )
	{
		return propertyMap.containsKey( name.toUpperCase() );
	}

	public int addPropertiesFromFile( String propFile )
	{
		try
		{
			InputStream input = null;
			
			File file = new File( propFile );
			if( file.exists() )
			{
				input = new FileInputStream( file );
			}
			else if( propFile.toLowerCase().startsWith( "http://" ) || propFile.toLowerCase().startsWith( "https://" ))
			{
			   UrlWsdlLoader loader = new UrlWsdlLoader( propFile );
			   loader.setUseWorker( false );
			   input = loader.load();
			}

			Properties properties = new Properties();
			properties.load( input );
			
			for( Object key : properties.keySet() )
			{
				String name = key.toString();
				if( !hasProperty( name ))
					addProperty( name ).setValue( properties.getProperty( name ) );
				else
					setPropertyValue( name, properties.getProperty( name ) );
			}
			
			return properties.size();
		}
		catch( Exception e )
		{
			SoapUI.logError( e );
		}
		
		return 0;
	}

	public ModelItem getModelItem()
	{
		return modelItem;
	}
}
